@@ -422,7 +422,7 @@ class Blueprint:
 
             def get_unique_linspace(gene):
                 genes = self._bp.genes[gene].get_linspace(n_dict[gene])
-                return np.unique(genes)
+                return pd.unique(genes)
 
             ranges = [get_unique_linspace(gene) for gene in self._bp._dict]
             data = itertools.product(*ranges)
